from ..exceptions import EmailActivityTrackerNotFoundException
from ..models import EmailActivityTracker, EmailLog, EventLog


class AbstractEmailProvider(object):
    provider = None

    @classmethod
    def send_email(cls, email_log: EmailLog):
        raise NotImplementedError

    @classmethod
    def parse_send_email_response_for_email_log(cls, response):
        raise NotImplementedError

    @classmethod
    def parse_event_webhook(cls, event_info):
        raise NotImplementedError

    @classmethod
    def handle_send_email_response(cls, response, email_log):
        email_status, parsed_response = cls.parse_send_email_response_for_email_log(
            response
        )
        if email_status == EmailLog.EMAIL_STATUS_SENT:
            email_log.update_fields(dispatch_status=email_status)

            for recipient_data in parsed_response:
                recipient_data["email_log"] = email_log
                EmailActivityTracker.track_recipient(recipient_data)

        elif email_status == EmailLog.EMAIL_STATUS_FAILED:
            email_log.update_fields(
                dispatch_status=email_status, error_info=parsed_response
            )

    # Handle this via creating a Celery task that retries at most 5 times
    # Figure out how to reference celery @app.task inside a package

    @classmethod
    def handle_event_webhook(cls, event_info):
        message_id, parsed_event_data = cls.parse_event_webhook(event_info)

        email_activity_tracker = EmailActivityTracker.get_by_message_id(message_id)
        if email_activity_tracker is None:
            # raise a normal exception here
            raise EmailActivityTrackerNotFoundException()

        parsed_event_data["email_activity_tracker"] = email_activity_tracker
        event_log = EventLog.add_new_event_log(parsed_event_data)

        email_activity_tracker.update_fields_on_event(event_log.event_type)



from django.contrib.postgres.fields import JSONField, ArrayField
from django.db import models
from django.db.models import F
from .abstract_models import AbstractModel
from .constants import EMAIL_PROVIDER_CHOICES


class EmailLog(AbstractModel):

    EMAIL_STATUS_QUEUED = "queued"
    EMAIL_STATUS_SENT = "sent"
    EMAIL_STATUS_FAILED = "failed"

    EMAIL_DISPATCH_STATUS_CHOICES = (
        (EMAIL_STATUS_SENT, "Sent"),
        (EMAIL_STATUS_QUEUED, "Queued"),
        (EMAIL_STATUS_FAILED, "Failed"),
    )

    email_provider = models.CharField(
        max_length=32,
        choices=EMAIL_PROVIDER_CHOICES,
        help_text="Name of the email provider used to send email",
    )

    from_email = models.EmailField(help_text="Email address of the sender")
    from_name = models.CharField(max_length=200, null=True, blank=True, help_text="Name of the sender")
    to_emails = ArrayField(
        models.EmailField(), help_text="List of email addresses of primary recipients",
    )
    cc_emails = ArrayField(
        models.EmailField(),
        null=True,
        help_text="List of email addresses of cc'ed recipients",
    )
    bcc_emails = ArrayField(
        models.EmailField(),
        null=True,
        help_text="List of email addresses of bcc'ed recipients",
    )

    subject = models.TextField(help_text="Subject of the email")
    body = models.TextField(null=True, help_text="Body of the email")
    template_id = models.CharField(
        max_length=64,
        null=True,
        help_text="Template Id of the template used to send email",
    )
    template_dynamic_data = JSONField(
        null=True,
        help_text="Dictionary containing values for " "dynamic variables in template",
    )

    dispatch_status = models.CharField(
        max_length=16,
        choices=EMAIL_DISPATCH_STATUS_CHOICES,
        default=EMAIL_STATUS_QUEUED,
        help_text="Status of the message whether it is not sent or sent",
    )

    error_info = JSONField(
        null=True, help_text="Details of error when message sending is failed"
    )

    def __str__(self):
        return f"To: {self.to_emails} Subject:{self.subject}"

    @classmethod
    def create_log(
        cls,
        email_provider,
        from_email,
        from_name,
        to_emails,
        cc_emails,
        bcc_emails,
        subject,
        body,
        template_id,
        template_dynamic_data,
    ):
        email_log = cls.objects.create(
            email_provider=email_provider,
            from_email=from_email,
            from_name=from_name,
            to_emails=to_emails,
            cc_emails=cc_emails,
            bcc_emails=bcc_emails,
            subject=subject,
            body=body,
            template_id=template_id,
            template_dynamic_data=template_dynamic_data,
        )

        return email_log


class EmailActivityTracker(AbstractModel):
    TO_RECIPIENT_TYPE = "to"
    CC_RECIPIENT_TYPE = "cc"
    BCC_RECIPIENT_TYPE = "bcc"

    RECIPIENT_TYPE_CHOICES = (
        (TO_RECIPIENT_TYPE, "To"),
        (CC_RECIPIENT_TYPE, "Cc"),
        (BCC_RECIPIENT_TYPE, "Bcc"),
    )

    SENT_EMAIL_STATUS_QUEUED = "queued"
    SENT_EMAIL_STATUS_DELIVERED = "delivered"
    SENT_EMAIL_STATUS_SPAMMED = "spammed"
    SENT_EMAIL_STATUS_SOFT_BOUNCED = "soft_bounced"
    SENT_EMAIL_STATUS_HARD_BOUNCED = "hard_bounced"

    SENT_EMAIL_STATUS_CHOICES = (
        (SENT_EMAIL_STATUS_QUEUED, "Queued"),
        (SENT_EMAIL_STATUS_DELIVERED, "Delivered"),
        (SENT_EMAIL_STATUS_SPAMMED, "Spammed"),
        (SENT_EMAIL_STATUS_SOFT_BOUNCED, "Soft Bounced"),
        (SENT_EMAIL_STATUS_HARD_BOUNCED, "Hard Bounced"),
    )

    email_log = models.ForeignKey(
        EmailLog,
        on_delete=models.PROTECT,
        help_text="Email info of the email which is being tracked",
    )
    recipient_type = models.CharField(
        max_length=3,
        choices=RECIPIENT_TYPE_CHOICES,
        help_text="Type of recipient to whom this email was sent",
    )
    email_address = models.EmailField(help_text="Email address of the recipient")

    message_id = models.CharField(
        max_length=128,
        unique=True,
        help_text="Unique identifier of the message which is being tracked",
    )

    open_count = models.IntegerField(default=0, help_text="Open count of the email")
    click_count = models.IntegerField(default=0, help_text="Click count of the email")

    email_status = models.CharField(
        max_length=16,
        choices=SENT_EMAIL_STATUS_CHOICES,
        default=SENT_EMAIL_STATUS_QUEUED,
        help_text="Status of the email after it is dispatched.",
    )

    def __str__(self):
        return self.email_address

    @classmethod
    def get_by_message_id(cls, message_id):
        try:
            return cls.objects.get(message_id=message_id)
        except cls.DoesNotExist:
            return None

    @classmethod
    def track_recipient(cls, recipient_data: dict):
        cls.objects.create(**recipient_data)

    def update_fields_on_event(self, event_type):
        event_type_vs_update_fields = {
            EventLog.EMAIL_OPENED_EVENT_TYPE: {"open_count": F("open_count") + 1},
            EventLog.EMAIL_CLICKED_EVENT_TYPE: {"click_count": F("click_count") + 1},
            EventLog.EMAIL_DELIVERED_EVENT_TYPE: {
                "email_status": EmailActivityTracker.SENT_EMAIL_STATUS_DELIVERED
            },
            EventLog.EMAIL_SOFT_BOUNCED_EVENT_TYPE: {
                "email_status": EmailActivityTracker.SENT_EMAIL_STATUS_SOFT_BOUNCED
            },
            EventLog.EMAIL_HARD_BOUNCED_EVENT_TYPE: {
                "email_status": EmailActivityTracker.SENT_EMAIL_STATUS_HARD_BOUNCED
            },
        }

        self.update_fields(**event_type_vs_update_fields[event_type])


class EventLog(AbstractModel):
    EMAIL_DELIVERED_EVENT_TYPE = "delivered"
    EMAIL_OPENED_EVENT_TYPE = "opened"
    EMAIL_CLICKED_EVENT_TYPE = "clicked"
    EMAIL_SPAMMED_EVENT_TYPE = "spammed"
    EMAIL_SOFT_BOUNCED_EVENT_TYPE = "soft_bounced"
    EMAIL_HARD_BOUNCED_EVENT_TYPE = "hard_bounced"

    EVENT_TYPE_CHOICES = (
        (EMAIL_DELIVERED_EVENT_TYPE, "Delivered"),
        (EMAIL_OPENED_EVENT_TYPE, "Opened"),
        (EMAIL_CLICKED_EVENT_TYPE, "Clicked"),
        (EMAIL_SPAMMED_EVENT_TYPE, "Spammed"),
        (EMAIL_SOFT_BOUNCED_EVENT_TYPE, "Soft Bounced"),
        (EMAIL_HARD_BOUNCED_EVENT_TYPE, "Hard bounced"),
    )

    email_activity_tracker = models.ForeignKey(
        EmailActivityTracker,
        on_delete=models.PROTECT,
        help_text="The email log to which this event belongs to",
    )
    event_type = models.CharField(
        max_length=16,
        choices=EVENT_TYPE_CHOICES,
        help_text="Type of the event which is being logged",
    )
    event_info = JSONField(help_text="Stores info about the event")

    event_at = models.DateTimeField(help_text="The time at which this event occurred")

    def __str__(self):
        return f"{self.event_at}: {self.event_type}"

    @classmethod
    def add_new_event_log(cls, event_info):
        return cls.objects.create(**event_info)



from django.http import HttpResponse
from rest_framework import status
from rest_framework.views import APIView

from .tasks import handle_webhook


class EmailEventWebhookView(APIView):
    authentication_classes = ()
    permission_classes = ()

    # Not using serializer here as the fields are dynamic in the json provided by webhook.
    def post(self, request, email_provider):
        # Using request.body instead of request.data here because keys in the dict are not
        # preserved in original format in request.data
        handle_webhook.apply_async((request.body.decode('utf-8'), email_provider))

        return HttpResponse(status=status.HTTP_200_OK)



import logging
from datetime import datetime

import pytz
from django.conf import settings
from mailjet_rest import Client

from ..constants import EMAIL_PROVIDER_MAILJET
from ..models import EventLog, EmailActivityTracker, EmailLog
from ..providers.abstract import AbstractEmailProvider

logger = logging.getLogger(__name__)


class MailjetEmailProvider(AbstractEmailProvider):

    provider = EMAIL_PROVIDER_MAILJET
    sdk_client = Client(
        auth=(settings.MAILJET_API_KEY, settings.MAILJET_SECRET_KEY),
        version="v3.1",
    )

    @classmethod
    def _create_email_data(cls, email_log):
        message = {
            "From": {"Email": email_log.from_email, "Name": email_log.from_name or email_log.from_email},
            "Subject": email_log.subject,
            "To": [{"Email": to_email} for to_email in email_log.to_emails],
        }

        if email_log.body:
            message["HTMLPart"] = email_log.body

        if email_log.cc_emails:
            message["Cc"] = [{"Email": cc_email} for cc_email in email_log.cc_emails]

        if email_log.bcc_emails:
            message["Bcc"] = [
                {"Email": bcc_email} for bcc_email in email_log.bcc_emails
            ]

        if email_log.template_id:
            message["TemplateID"] = email_log.template_id
            message["TemplateLanguage"] = True
            message["Variables"] = email_log.template_dynamic_data

        return {"Messages": [message]}

    @classmethod
    def send_email(cls, email_log):
        data = cls._create_email_data(email_log)
        response = cls.sdk_client.send.create(data=data)

        return response

    @classmethod
    def parse_send_email_response_for_email_log(cls, response):
        response_data = response.json()

        if response.ok:
            recipients_response_data = response_data["Messages"][0]

            recipient_vs_type = {
                "To": EmailActivityTracker.TO_RECIPIENT_TYPE,
                "Cc": EmailActivityTracker.CC_RECIPIENT_TYPE,
                "Bcc": EmailActivityTracker.BCC_RECIPIENT_TYPE,
            }

            parsed_response = []
            for recipient in recipient_vs_type.keys():
                parsed_response += [
                    {
                        "recipient_type": recipient_vs_type[recipient],
                        "email_address": recipient_response_data["Email"],
                        "message_id": str(recipient_response_data["MessageID"]),
                    }
                    for recipient_response_data in recipients_response_data[recipient]
                ]

            return EmailLog.EMAIL_STATUS_SENT, parsed_response

        else:
            error_info = {"error": response_data}

            logger.exception(
                "Mailjet: Could not send email",
                extra={"status code": response.status_code, "errors": error_info},
            )

            return EmailLog.EMAIL_STATUS_FAILED, error_info

    @classmethod
    def parse_event_webhook(cls, event_info):
        message_id = str(event_info["MessageID"])

        event_vs_event_type = {
            "open": EventLog.EMAIL_OPENED_EVENT_TYPE,
            "click": EventLog.EMAIL_CLICKED_EVENT_TYPE,
            "spam": EventLog.EMAIL_SPAMMED_EVENT_TYPE,
            "sent": EventLog.EMAIL_DELIVERED_EVENT_TYPE,
            "bounce": EventLog.EMAIL_HARD_BOUNCED_EVENT_TYPE
            if event_info.get("hard_bounce")
            else EventLog.EMAIL_SOFT_BOUNCED_EVENT_TYPE,
        }

        event_data = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": event_vs_event_type[event_info["event"]],
        }

        return message_id, event_data



from unittest import mock

from django.test import TestCase
from django.conf import settings

from ..constants import EMAIL_PROVIDER_MAILJET
from ..models import EmailLog
from ..providers.mailjet import MailjetEmailProvider
from ..services import EmailService
from ..tests.factories import EmailLogFactory


class EmailServiceTestCase(TestCase):
    def test_get_provider_class_for_provider(self):
        provider = EMAIL_PROVIDER_MAILJET
        provider_class = EmailService._get_provider_class_for_provider(provider)

        self.assertEqual(provider_class, MailjetEmailProvider)

        provider = "test_provider"
        with self.assertRaises(ValueError):
            EmailService._get_provider_class_for_provider(provider)

    def test_send_email_of_mailjet(self):
        to_emails = ["to_email@example.com"]
        cc_emails = ["cc_email@example.com"]
        bcc_emails = ["bcc_email@example.com"]
        subject = "Test Email"
        response = {"MessageId": "123456"}
        body = "test body"
        template_id = "test_template_id"
        template_dynamic_data = {"name": "test_name"}
        email_log = EmailLogFactory()
        from_email = settings.DEFAULT_FROM_EMAIL
        from_name = settings.DEFAULT_FROM_NAME
        email_provider = EMAIL_PROVIDER_MAILJET

        with mock.patch.object(
            EmailService, "_get_provider_class_for_provider"
        ) as mocked_get_provider_class_for_provider, mock.patch.object(
            EmailLog, "create_log"
        ) as mocked_create_log, mock.patch.object(
            MailjetEmailProvider, "send_email"
        ) as mocked_mailjet_send_email, mock.patch.object(
            MailjetEmailProvider, "handle_send_email_response"
        ) as mocked_handle_send_email_response:
            mocked_get_provider_class_for_provider.return_value = MailjetEmailProvider
            mocked_create_log.return_value = email_log
            mocked_mailjet_send_email.return_value = response

            EmailService.send_email(
                to_emails,
                subject,
                cc_emails,
                bcc_emails,
                body,
                template_id,
                template_dynamic_data,
                from_email,
                from_name,
                email_provider,
            )

            mocked_get_provider_class_for_provider.assert_called_with(email_provider)
            mocked_create_log.assert_called_with(
                EMAIL_PROVIDER_MAILJET,
                from_email,
                from_name,
                to_emails,
                cc_emails,
                bcc_emails,
                subject,
                body,
                template_id,
                template_dynamic_data,
            )
            mocked_mailjet_send_email.assert_called_with(email_log)
            mocked_handle_send_email_response.called_with(response, email_log)

    def test_handle_event_webhook_of_mailjet(self):
        event_info = {"event": "sent"}

        with mock.patch.object(
            EmailService, "_get_provider_class_for_provider"
        ) as mocked_get_provider_class, mock.patch.object(
            MailjetEmailProvider, "handle_event_webhook"
        ) as mocked_mailjet_handle_event_webhook:
            mocked_get_provider_class.return_value = MailjetEmailProvider

            EmailService.handle_event_webhook(EMAIL_PROVIDER_MAILJET, event_info)

            mocked_get_provider_class.assert_called_with(EMAIL_PROVIDER_MAILJET)
            mocked_mailjet_handle_event_webhook.assert_called_with(event_info)



from datetime import datetime

import pytz
from django.conf import settings
from django.test import TestCase

from ..constants import DEFAULT_EMAIL_PROVIDER
from ..models import EmailLog, EmailActivityTracker, EventLog
from ..tests.factories import (
    EmailLogFactory,
    EmailActivityTrackerFactory,
)


class EmailLogModelTestCase(TestCase):
    def test_create_log(self):

        email_provider = DEFAULT_EMAIL_PROVIDER
        from_email = settings.DEFAULT_FROM_NAME
        from_name = "Tony Stark"
        to_emails = ["to_email@example.com"]
        cc_emails = ["cc_email@example.com"]
        bcc_emails = ["bcc_email@example.com"]
        subject = "Test Email"
        body = "Test body"
        template_id = "12345678"
        template_dynamic_data = {"name": "example_name"}

        email_log = EmailLog.create_log(
            email_provider,
            from_email,
            from_name,
            to_emails,
            cc_emails,
            bcc_emails,
            subject,
            body,
            template_id,
            template_dynamic_data,
        )
        self.assertTupleEqual(
            (
                email_log.email_provider,
                email_log.from_email,
                email_log.to_emails,
                email_log.cc_emails,
                email_log.bcc_emails,
                email_log.subject,
                email_log.body,
                email_log.template_id,
                email_log.template_dynamic_data,
                email_log.dispatch_status,
            ),
            (
                email_provider,
                from_email,
                to_emails,
                cc_emails,
                bcc_emails,
                subject,
                body,
                template_id,
                template_dynamic_data,
                EmailLog.EMAIL_STATUS_QUEUED,
            ),
        )


class EmailActivityTrackerModelTestCase(TestCase):
    def setUp(self):
        self.email_log = EmailLogFactory()
        self.email_activity_tracker = EmailActivityTrackerFactory()

    def test_get_by_message_id(self):
        message_id = "test_message_id"
        self.assertIsNone(EmailActivityTracker.get_by_message_id(message_id))

        self.assertEqual(
            self.email_activity_tracker,
            EmailActivityTracker.get_by_message_id(
                self.email_activity_tracker.message_id
            ),
        )

    def test_track_recipient(self):
        recipient_data = {
            "email_log": self.email_log,
            "recipient_type": EmailActivityTracker.TO_RECIPIENT_TYPE,
            "email_address": "noreply@example.com",
            "message_id": "123456789",
        }

        EmailActivityTracker.track_recipient(recipient_data)
        email_activity_tracker = EmailActivityTracker.get_by_message_id(
            recipient_data["message_id"]
        )
        self.assertTupleEqual(
            (
                email_activity_tracker.email_log,
                email_activity_tracker.recipient_type,
                email_activity_tracker.email_address,
                email_activity_tracker.message_id,
                email_activity_tracker.email_status,
                email_activity_tracker.open_count,
                email_activity_tracker.click_count,
            ),
            (
                recipient_data["email_log"],
                recipient_data["recipient_type"],
                recipient_data["email_address"],
                recipient_data["message_id"],
                EmailActivityTracker.SENT_EMAIL_STATUS_QUEUED,
                0,
                0,
            ),
        )

    def test_update_fields_on_open_event(self):
        old_open_count = self.email_activity_tracker.open_count

        self.email_activity_tracker.update_fields_on_event(
            EventLog.EMAIL_OPENED_EVENT_TYPE
        )
        self.email_activity_tracker.refresh_from_db()

        self.assertEqual(self.email_activity_tracker.open_count, old_open_count + 1)

    def test_update_fields_on_click_event(self):
        old_click_count = self.email_activity_tracker.click_count

        self.email_activity_tracker.update_fields_on_event(
            EventLog.EMAIL_CLICKED_EVENT_TYPE
        )
        self.email_activity_tracker.refresh_from_db()

        self.assertEqual(self.email_activity_tracker.click_count, old_click_count + 1)

    def test_update_fields_on_sent_event(self):
        self.email_activity_tracker.update_fields_on_event(
            EventLog.EMAIL_DELIVERED_EVENT_TYPE
        )

        self.assertEqual(
            self.email_activity_tracker.email_status,
            EmailActivityTracker.SENT_EMAIL_STATUS_DELIVERED,
        )

    def test_update_fields_on_soft_bounced_event(self):
        self.email_activity_tracker.update_fields_on_event(
            EventLog.EMAIL_SOFT_BOUNCED_EVENT_TYPE
        )

        self.assertEqual(
            self.email_activity_tracker.email_status,
            EmailActivityTracker.SENT_EMAIL_STATUS_SOFT_BOUNCED,
        )

    def test_update_fields_on_hard_bounced(self):
        self.email_activity_tracker.update_fields_on_event(
            EventLog.EMAIL_HARD_BOUNCED_EVENT_TYPE
        )

        self.assertEqual(
            self.email_activity_tracker.email_status,
            EmailActivityTracker.SENT_EMAIL_STATUS_HARD_BOUNCED,
        )


class EventLogModelTestCase(TestCase):
    def setUp(self):
        self.email_activity_tracker = EmailActivityTrackerFactory()

    def test_add_new_event_log(self):
        event_data = {
            "email_activity_tracker": self.email_activity_tracker,
            "event_at": pytz.UTC.localize(datetime.now()),
            "event_type": EventLog.EMAIL_DELIVERED_EVENT_TYPE,
            "event_info": {"event": "delivered"},
        }

        EventLog.add_new_event_log(event_data)
        event_log = EventLog.objects.get(
            email_activity_tracker=self.email_activity_tracker,
            event_at=event_data["event_at"],
        )

        self.assertTupleEqual(
            (event_log.event_type, event_log.event_info),
            (event_data["event_type"], event_data["event_info"]),
        )



import json
from datetime import datetime
from unittest import mock

import pytz
from django.test import TestCase
from requests.models import Response
from rest_framework import status

from ..exceptions import EmailActivityTrackerNotFoundException
from ..models import EmailActivityTracker, EmailLog, EventLog
from ..providers.mailjet import MailjetEmailProvider
from ..tests.factories import (
    EmailLogFactory,
    EmailActivityTrackerFactory,
    EventLogFactory,
)


class MailjetEmailProviderTestCase(TestCase):
    def setUp(self):
        self.email_log = EmailLogFactory()
        self.email_activity_tracker = EmailActivityTrackerFactory()
        self.event_log = EventLogFactory()

    def test_create_email_data(self):
        data = {
            "Messages": [
                {
                    "From": {"Email": self.email_log.from_email, "Name": self.email_log.from_name},
                    "To": [{"Email": self.email_log.to_emails[0]}],
                    "Cc": [{"Email": self.email_log.cc_emails[0]}],
                    "Bcc": [{"Email": self.email_log.bcc_emails[0]}],
                    "Subject": self.email_log.subject,
                    "HTMLPart": self.email_log.body,
                }
            ]
        }

        self.assertEqual(MailjetEmailProvider._create_email_data(self.email_log), data)

    def test_create_email_data_with_template(self):
        self.email_log.update_fields(
            body=None,
            cc_emails=None,
            bcc_emails=None,
            template_id="test_id",
            template_dynamic_data={"variable": "test_variable"},
        )

        data = {
            "Messages": [
                {
                    "From": {"Email": self.email_log.from_email, "Name": self.email_log.from_name},
                    "To": [{"Email": self.email_log.to_emails[0]}],
                    "Subject": self.email_log.subject,
                    "TemplateID": self.email_log.template_id,
                    "TemplateLanguage": True,
                    "Variables": self.email_log.template_dynamic_data,
                }
            ]
        }

        self.assertDictEqual(
            MailjetEmailProvider._create_email_data(self.email_log), data
        )

    def test_parse_sent_email_response_for_email_log(self):
        api_sent_email_response = {
            "Messages": [
                {
                    "Status": "success",
                    "To": [
                        {
                            "Email": "passenger1@mailjet.com",
                            "MessageUUID": "123",
                            "MessageID": 456,
                            "MessageHref": "https://api.mailjet.com/v3/message/456",
                        },
                    ],
                    "Cc": [
                        {
                            "Email": "copilot@mailjet.com",
                            "MessageUUID": "125",
                            "MessageID": 458,
                            "MessageHref": "https://api.mailjet.com/v3/message/458",
                        }
                    ],
                    "Bcc": [
                        {
                            "Email": "air-traffic-control@mailjet.com",
                            "MessageUUID": "126",
                            "MessageID": 459,
                            "MessageHref": "https://api.mailjet.com/v3/message/459",
                        }
                    ],
                }
            ]
        }

        parsed_sent_email_response = [
            {
                "recipient_type": EmailActivityTracker.TO_RECIPIENT_TYPE,
                "email_address": "passenger1@mailjet.com",
                "message_id": str(456),
            },
            {
                "recipient_type": EmailActivityTracker.CC_RECIPIENT_TYPE,
                "email_address": "copilot@mailjet.com",
                "message_id": str(458),
            },
            {
                "recipient_type": EmailActivityTracker.BCC_RECIPIENT_TYPE,
                "email_address": "air-traffic-control@mailjet.com",
                "message_id": str(459),
            },
        ]

        sent_email_response = Response()
        sent_email_response._content = json.dumps(api_sent_email_response).encode(
            "utf-8"
        )
        sent_email_response.status_code = status.HTTP_200_OK

        self.assertTupleEqual(
            MailjetEmailProvider.parse_send_email_response_for_email_log(
                sent_email_response
            ),
            (EmailLog.EMAIL_STATUS_SENT, parsed_sent_email_response),
        )

    def test_parse_send_email_error_response_for_email_log(self):
        api_send_email_error_response = {
            "Messages": [
                {
                    "Status": "success",
                    "Errors": [
                        {
                            "ErrorIdentifier": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
                            "ErrorCode": "send-0010",
                            "StatusCode": 400,
                            "ErrorMessage": 'Template ID "123456789" doesn\'t exist for your account.',
                            "ErrorRelatedTo": "TemplateID",
                        }
                    ],
                }
            ]
        }

        parsed_send_email_error_response = {
            "error": {
                "Messages": [
                    {
                        "Status": "success",
                        "Errors": [
                            {
                                "ErrorIdentifier": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
                                "ErrorCode": "send-0010",
                                "StatusCode": 400,
                                "ErrorMessage": 'Template ID "123456789" doesn\'t exist for your account.',
                                "ErrorRelatedTo": "TemplateID",
                            }
                        ],
                    }
                ]
            }
        }

        send_email_error_response = Response()
        send_email_error_response._content = json.dumps(
            api_send_email_error_response
        ).encode("utf-8")
        send_email_error_response.status_code = status.HTTP_400_BAD_REQUEST

        self.assertTupleEqual(
            MailjetEmailProvider.parse_send_email_response_for_email_log(
                send_email_error_response
            ),
            (EmailLog.EMAIL_STATUS_FAILED, parsed_send_email_error_response),
        )

    def test_parse_event_webhook_for_email_sent(self):
        event_info = {
            "event": "sent",
            "time": 1433103519,
            "MessageID": 19421777396190490,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "api@mailjet.com",
            "mj_campaign_id": 7173,
            "mj_contact_id": 320,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "ip": "127.0.0.1",
            "geo": "US",
            "agent": "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox/11.0",
        }

        parsed_event_info = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": EventLog.EMAIL_DELIVERED_EVENT_TYPE,
        }

        self.assertTupleEqual(
            MailjetEmailProvider.parse_event_webhook(event_info),
            (str(event_info["MessageID"]), parsed_event_info),
        )

    def test_parse_event_webhook_for_email_opened(self):
        event_info = {
            "event": "open",
            "time": 1433103519,
            "MessageID": 19421777396190490,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "api@mailjet.com",
            "mj_campaign_id": 7173,
            "mj_contact_id": 320,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "ip": "127.0.0.1",
            "geo": "US",
            "agent": "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox/11.0",
        }

        parsed_event_info = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": EventLog.EMAIL_OPENED_EVENT_TYPE,
        }

        self.assertTupleEqual(
            MailjetEmailProvider.parse_event_webhook(event_info),
            (str(event_info["MessageID"]), parsed_event_info),
        )

    def test_parse_event_webhook_for_email_clicked_event(self):
        event_info = {
            "event": "click",
            "time": 1433334653,
            "MessageID": 19421777836302490,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "api@mailjet.com",
            "mj_campaign_id": 7272,
            "mj_contact_id": 4,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "url": "https://mailjet.com",
            "ip": "127.0.0.1",
            "geo": "FR",
            "agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_0) AppleWebKit/537.36",
        }

        parsed_event_info = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": EventLog.EMAIL_CLICKED_EVENT_TYPE,
        }

        self.assertTupleEqual(
            MailjetEmailProvider.parse_event_webhook(event_info),
            (str(event_info["MessageID"]), parsed_event_info),
        )

    def test_parse_event_webhook_for_email_hard_bounced_event(self):
        event_info = {
            "event": "bounce",
            "time": 1430812195,
            "MessageID": 13792286917004336,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "bounce@mailjet.com",
            "mj_campaign_id": 0,
            "mj_contact_id": 0,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "blocked": False,
            "hard_bounce": True,
            "error_related_to": "recipient",
            "error": "user unknown",
        }

        parsed_event_info = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": EventLog.EMAIL_HARD_BOUNCED_EVENT_TYPE,
        }

        self.assertTupleEqual(
            MailjetEmailProvider.parse_event_webhook(event_info),
            (str(event_info["MessageID"]), parsed_event_info),
        )

    def test_parse_event_webhook_for_email_soft_bounced_event(self):
        event_info = {
            "event": "bounce",
            "time": 1430812195,
            "MessageID": 13792286917004336,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "bounce@mailjet.com",
            "mj_campaign_id": 0,
            "mj_contact_id": 0,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "blocked": False,
            "error_related_to": "recipient",
            "error": "user unknown",
        }

        parsed_event_info = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": EventLog.EMAIL_SOFT_BOUNCED_EVENT_TYPE,
        }

        self.assertTupleEqual(
            MailjetEmailProvider.parse_event_webhook(event_info),
            (str(event_info["MessageID"]), parsed_event_info),
        )

    def test_parse_event_webhook_for_spam_event(self):
        event_info = {
            "event": "spam",
            "time": 1430812195,
            "MessageID": 13792286917004336,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "bounce@mailjet.com",
            "mj_campaign_id": 0,
            "mj_contact_id": 0,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "source": "JMRPP",
        }
        parsed_event_info = {
            "event_at": pytz.UTC.localize(
                datetime.utcfromtimestamp(event_info["time"])
            ),
            "event_info": event_info,
            "event_type": EventLog.EMAIL_SPAMMED_EVENT_TYPE,
        }

        self.assertTupleEqual(
            MailjetEmailProvider.parse_event_webhook(event_info),
            (str(event_info["MessageID"]), parsed_event_info),
        )

    def test_parse_event_webhook_for_not_defined_email_event(self):
        event_info = {
            "event": "not_defined",
            "time": 1433103519,
            "MessageID": 19421777396190490,
            "Message_GUID": "1ab23cd4-e567-8901-2345-6789f0gh1i2j",
            "email": "api@mailjet.com",
            "mj_campaign_id": 7173,
            "mj_contact_id": 320,
            "customcampaign": "",
            "CustomID": "helloworld",
            "Payload": "",
            "ip": "127.0.0.1",
            "geo": "US",
            "agent": "Mozilla/5.0 (Windows NT 5.1; rv:11.0) Gecko Firefox/11.0",
        }

        with self.assertRaises(KeyError):
            MailjetEmailProvider.parse_event_webhook(event_info)

    def test_send_email(self):
        data = {"Email": "test@example.com"}

        with mock.patch.object(
            MailjetEmailProvider, "_create_email_data"
        ) as mocked_create_email_data, mock.patch(
            "django_email.providers.mailjet.MailjetEmailProvider.sdk_client"
        ) as mailjet_sdk_client:
            mocked_create_email_data.return_value = data
            sdk_send_method = mailjet_sdk_client.send.create

            MailjetEmailProvider.send_email(self.email_log)

            mocked_create_email_data.assert_called_with(self.email_log)
            sdk_send_method.assert_called_with(data=data)

    def test_handle_send_email_response(self):
        response_from_provider = [{"To": {"Message_Id": "1234"}}]
        parsed_response = [{"message_id": "1234"}]
        recipient_response = {"message_id": "1234", "email_log": self.email_log}

        with mock.patch.object(
            MailjetEmailProvider, "parse_send_email_response_for_email_log"
        ) as mocked_parse_send_email_response, mock.patch.object(
            EmailActivityTracker, "track_recipient"
        ) as mocked_track_recipient, mock.patch.object(
            EmailLog, "update_fields"
        ) as mocked_email_log_update_fields:
            mocked_parse_send_email_response.return_value = (
                EmailLog.EMAIL_STATUS_SENT,
                parsed_response,
            )

            MailjetEmailProvider.handle_send_email_response(
                response_from_provider, self.email_log
            )

            mocked_parse_send_email_response.assert_called_with(response_from_provider)
            mocked_email_log_update_fields.assert_called_with(
                dispatch_status=EmailLog.EMAIL_STATUS_SENT
            )
            mocked_track_recipient.assert_has_calls([mock.call(recipient_response)])

            # Below is the test for the scenario when error is received as the response
            mocked_parse_send_email_response.return_value = (
                EmailLog.EMAIL_STATUS_FAILED,
                parsed_response,
            )

            MailjetEmailProvider.handle_send_email_response(
                response_from_provider, self.email_log
            )

            mocked_parse_send_email_response.assert_called_with(response_from_provider)
            mocked_email_log_update_fields.assert_called_with(
                dispatch_status=EmailLog.EMAIL_STATUS_FAILED, error_info=parsed_response
            )

            mocked_parse_send_email_response.return_value = (
                "Undefined Status",
                parsed_response,
            )

            MailjetEmailProvider.handle_send_email_response(
                response_from_provider, self.email_log
            )
            # Call count of update fields would remain 2 as email log fields would not be updated for undefined
            # email status
            self.assertEqual(mocked_email_log_update_fields.call_count, 2)

    def test_handle_event_webhook(self):
        event_info = {"event": "sent", "MessageId": "12345"}
        parsed_event_data = {"event": EventLog.EMAIL_DELIVERED_EVENT_TYPE}
        event_data_for_model = {
            **parsed_event_data,
            "email_activity_tracker": self.email_activity_tracker,
        }

        with mock.patch.object(
            MailjetEmailProvider, "parse_event_webhook"
        ) as mocked_abstract_parse_event_webhook, mock.patch.object(
            EmailActivityTracker, "get_by_message_id"
        ) as mocked_get_by_message_id, mock.patch.object(
            EventLog, "add_new_event_log"
        ) as mocked_add_event_log, mock.patch.object(
            EmailActivityTracker, "update_fields_on_event"
        ) as mocked_email_activity_tracker_update_fields:

            mocked_abstract_parse_event_webhook.return_value = (
                event_info["MessageId"],
                parsed_event_data,
            )
            mocked_get_by_message_id.return_value = self.email_activity_tracker
            mocked_add_event_log.return_value = self.event_log

            MailjetEmailProvider.handle_event_webhook(event_info)

            mocked_abstract_parse_event_webhook.assert_called_with(event_info)
            mocked_get_by_message_id.assert_called_with(event_info["MessageId"])
            self.assertEqual(event_data_for_model, parsed_event_data)
            mocked_email_activity_tracker_update_fields.assert_called_with(
                self.event_log.event_type
            )

            # Below is the test for the scenario when no activity tracker is found in db and an
            # exception should be raised in that case
            with self.assertRaises(EmailActivityTrackerNotFoundException):
                mocked_abstract_parse_event_webhook.return_value = (
                    event_info["MessageId"],
                    parsed_event_data,
                )
                mocked_get_by_message_id.return_value = None
                mocked_add_event_log.return_value = self.event_log

                MailjetEmailProvider.handle_event_webhook(event_info)



# Generated by Django 3.0.5 on 2020-05-28 07:27

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_email", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="emailactivitytracker",
            name="is_active",
            field=models.BooleanField(
                default=True,
                help_text="Denotes if the object is active or not. Inactive objects behave similar to how a deleted "
                "object works.",
            ),
        ),
        migrations.AlterField(
            model_name="emaillog",
            name="is_active",
            field=models.BooleanField(
                default=True,
                help_text="Denotes if the object is active or not. Inactive objects behave similar to how a deleted "
                "object works.",
            ),
        ),
        migrations.AlterField(
            model_name="eventlog",
            name="is_active",
            field=models.BooleanField(
                default=True,
                help_text="Denotes if the object is active or not. Inactive objects behave similar to how a deleted "
                "object works.",
            ),
        ),
    ]



from django.test import TestCase
from django.urls import resolve, reverse

from ..constants import EMAIL_PROVIDER_MAILJET
from ..views import EmailEventWebhookView


class TestUrls(TestCase):
    def test_email_event_webhook_view(self):
        url = reverse(
            "email_event_webhook", kwargs={"email_provider": EMAIL_PROVIDER_MAILJET}
        )
        self.assertEqual(resolve(url).func.view_class, EmailEventWebhookView)



import json

from celery import shared_task

from .exceptions import EmailActivityTrackerNotFoundException
from .services import EmailService


@shared_task(
    autoretry_for=(EmailActivityTrackerNotFoundException,), default_retry_delay=60,
    retry_kwargs={'max_retries': 5}
)
def handle_webhook(request_body: str, email_provider):
    event_info = json.loads(request_body)
    EmailService.handle_event_webhook(email_provider, event_info)



from django.urls import path

from .views import EmailEventWebhookView

urlpatterns = [
    path(
        "<str:email_provider>/event/",
        EmailEventWebhookView.as_view(),
        name="email_event_webhook",
    ),
]



import logging

from django.conf import settings

from .constants import DEFAULT_EMAIL_PROVIDER
from .models import EmailLog
from .providers.abstract import AbstractEmailProvider

logger = logging.getLogger(__name__)


class EmailService(object):
    @classmethod
    def _get_provider_class_for_provider(cls, provider: str):
        for provider_class in AbstractEmailProvider.__subclasses__():
            if provider_class.provider == provider:
                return provider_class

        else:
            logger.warning(
                f"Unsupported email provider {provider} requested for service"
            )
            raise ValueError(
                f"Provider {provider} is not supported by the email module as of today."
            )

    @classmethod
    def send_email(
        cls,
        to_emails: list,
        subject,
        cc_emails: list = None,
        bcc_emails: list = None,
        body=None,
        template_id=None,
        template_dynamic_data: dict = None,
        from_email=settings.DEFAULT_FROM_EMAIL,
        from_name=settings.DEFAULT_FROM_NAME,
        email_provider=DEFAULT_EMAIL_PROVIDER,
    ):
        provider_class = cls._get_provider_class_for_provider(email_provider)

        email_log = EmailLog.create_log(
            email_provider,
            from_email,
            from_name,
            to_emails,
            cc_emails,
            bcc_emails,
            subject,
            body,
            template_id,
            template_dynamic_data,
        )

        response = provider_class.send_email(email_log)
        provider_class.handle_send_email_response(response, email_log)

    @classmethod
    def handle_event_webhook(cls, email_provider, event_info):
        provider_class = cls._get_provider_class_for_provider(email_provider)

        provider_class.handle_event_webhook(event_info)



# Generated by Django 3.0.5 on 2020-05-22 10:57

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="EmailActivityTracker",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="Time of creation of this object",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="Time of updation of this object"
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Denotes if the object is active or not."
                        "Inactive objects behave similar to how a deleted object works.",
                    ),
                ),
                (
                    "recipient_type",
                    models.CharField(
                        choices=[("to", "To"), ("cc", "Cc"), ("bcc", "Bcc")],
                        help_text="Type of recipient to whom this email was sent",
                        max_length=3,
                    ),
                ),
                (
                    "email_address",
                    models.EmailField(
                        help_text="Email address of the recipient", max_length=254
                    ),
                ),
                (
                    "message_id",
                    models.CharField(
                        help_text="Unique identifier of the message which is being tracked",
                        max_length=128,
                        unique=True,
                    ),
                ),
                (
                    "open_count",
                    models.IntegerField(default=0, help_text="Open count of the email"),
                ),
                (
                    "click_count",
                    models.IntegerField(
                        default=0, help_text="Click count of the email"
                    ),
                ),
                (
                    "email_status",
                    models.CharField(
                        choices=[
                            ("queued", "Queued"),
                            ("delivered", "Delivered"),
                            ("spammed", "Spammed"),
                            ("soft_bounced", "Soft Bounced"),
                            ("hard_bounced", "Hard Bounced"),
                        ],
                        default="queued",
                        help_text="Status of the email after it is dispatched.",
                        max_length=16,
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="EmailLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="Time of creation of this object",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="Time of updation of this object"
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Denotes if the object is active or not."
                        "Inactive objects behave similar to how a deleted object works.",
                    ),
                ),
                (
                    "email_provider",
                    models.CharField(
                        choices=[("mailjet", "Mailjet")],
                        help_text="Name of the email provider used to send email",
                        max_length=32,
                    ),
                ),
                (
                    "from_email",
                    models.EmailField(
                        help_text="Email address of the sender", max_length=254
                    ),
                ),
                (
                    "to_emails",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.EmailField(max_length=254),
                        help_text="List of email addresses of primary recipients",
                        size=None,
                    ),
                ),
                (
                    "cc_emails",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.EmailField(max_length=254),
                        help_text="List of email addresses of cc'ed recipients",
                        null=True,
                        size=None,
                    ),
                ),
                (
                    "bcc_emails",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.EmailField(max_length=254),
                        help_text="List of email addresses of bcc'ed recipients",
                        null=True,
                        size=None,
                    ),
                ),
                ("subject", models.TextField(help_text="Subject of the email")),
                ("body", models.TextField(help_text="Body of the email", null=True)),
                (
                    "template_id",
                    models.CharField(
                        help_text="Template Id of the template used to send email",
                        max_length=64,
                        null=True,
                    ),
                ),
                (
                    "template_dynamic_data",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="Dictionary containing values for dynamic variables in template",
                        null=True,
                    ),
                ),
                (
                    "dispatch_status",
                    models.CharField(
                        choices=[
                            ("sent", "Sent"),
                            ("queued", "Queued"),
                            ("failed", "Failed"),
                        ],
                        default="queued",
                        help_text="Status of the message whether it is not sent or sent",
                        max_length=16,
                    ),
                ),
                (
                    "error_info",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="Details of error when message sending is failed",
                        null=True,
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="EventLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        editable=False,
                        help_text="Time of creation of this object",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="Time of updation of this object"
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Denotes if the object is active or not."
                        "Inactive objects behave similar to how a deleted object works.",
                    ),
                ),
                (
                    "event_type",
                    models.CharField(
                        choices=[
                            ("delivered", "Delivered"),
                            ("opened", "Opened"),
                            ("clicked", "Clicked"),
                            ("spammed", "Spammed"),
                            ("soft_bounced", "Soft Bounced"),
                            ("hard_bounced", "Hard bounced"),
                        ],
                        help_text="Type of the event which is being logged",
                        max_length=16,
                    ),
                ),
                (
                    "event_info",
                    django.contrib.postgres.fields.jsonb.JSONField(
                        help_text="Stores info about the event"
                    ),
                ),
                (
                    "event_at",
                    models.DateTimeField(
                        help_text="The time at which this event occurred"
                    ),
                ),
                (
                    "email_activity_tracker",
                    models.ForeignKey(
                        help_text="The email log to which this event belongs to",
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_email.EmailActivityTracker",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.AddIndex(
            model_name="emaillog",
            index=models.Index(
                condition=models.Q(is_active=True),
                fields=["is_active"],
                name="emaillog_is_active",
            ),
        ),
        migrations.AddField(
            model_name="emailactivitytracker",
            name="email_log",
            field=models.ForeignKey(
                help_text="Email info of the email which is being tracked",
                on_delete=django.db.models.deletion.PROTECT,
                to="django_email.EmailLog",
            ),
        ),
        migrations.AddIndex(
            model_name="eventlog",
            index=models.Index(
                condition=models.Q(is_active=True),
                fields=["is_active"],
                name="eventlog_is_active",
            ),
        ),
        migrations.AddIndex(
            model_name="emailactivitytracker",
            index=models.Index(
                condition=models.Q(is_active=True),
                fields=["is_active"],
                name="emailactivitytracker_is_active",
            ),
        ),
    ]



import json
from unittest import mock

from django.urls import reverse
from django.conf import settings
from rest_framework import status
from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from ..constants import EMAIL_PROVIDER_MAILJET
from ..services import EmailService


class EmailEventWebhookViewTestCase(APITestCase):
    def setUp(self):
        self.client = APIClient()

    def test_email_event_webhook_view_POST(self):
        email_provider = EMAIL_PROVIDER_MAILJET
        event_webhook_url = reverse(
            "email_event_webhook", kwargs={"email_provider": email_provider}
        )
        event_info_dict = {"event": "sent"}
        event_info_json = json.loads(json.dumps(event_info_dict))

        with mock.patch('django_email.views.handle_webhook.apply_async') as mocked_handle_event_webhook:
            response = self.client.post(
                event_webhook_url, event_info_json, format="json"
            )

            mocked_handle_event_webhook.assert_called_with(
                ('{"event":"sent"}', email_provider)
            )
            self.assertEqual(response.status_code, status.HTTP_200_OK)



from setuptools import setup

install_requires = [
    'pytz>=2019.1', 'mailjet-rest==1.3.3', 'Django>=2.2.0', 'django-nested-admin==3.3.0',
    'djangorestframework>=3.9.0', 'celery>=4.0.0', 'requests>=2.0.0', 'Faker>=2.0.0',
    'factory-boy>=2.0.0'
]

setup(install_requires=install_requires, long_description_content_type='text/x-rst')



# Generated by Django 3.0.5 on 2020-07-14 16:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_email', '0002_auto_20200528_1257'),
    ]

    operations = [
        migrations.AddField(
            model_name='emaillog',
            name='from_name',
            field=models.CharField(blank=True, help_text='Name of the sender', max_length=200, null=True),
        ),
    ]



import factory
import pytz
from factory.django import DjangoModelFactory
from faker import Factory

from ..constants import EMAIL_PROVIDER_CHOICES
from ..models import EmailLog, EmailActivityTracker, EventLog

faker = Factory.create()


class EmailLogFactory(DjangoModelFactory):
    class Meta:
        model = EmailLog

    email_provider = factory.LazyAttribute(
        lambda _: faker.random_choices(elements=[x[0] for x in EMAIL_PROVIDER_CHOICES])[
            0
        ]
    )
    from_email = factory.LazyAttribute(lambda _: faker.email())
    from_name = factory.LazyAttribute(lambda _: faker.name())
    to_emails = factory.LazyAttribute(lambda _: [faker.email()])
    cc_emails = factory.LazyAttribute(lambda _: [faker.email()])
    bcc_emails = factory.LazyAttribute(lambda _: [faker.email()])
    subject = factory.LazyAttribute(lambda _: [faker.text(max_nb_chars=20)])
    body = factory.LazyAttribute(lambda _: [faker.text(max_nb_chars=40)])


class EmailActivityTrackerFactory(DjangoModelFactory):
    class Meta:
        model = EmailActivityTracker

    email_log = factory.SubFactory(EmailLogFactory)
    recipient_type = EmailActivityTracker.TO_RECIPIENT_TYPE
    email_address = factory.LazyAttribute(lambda _: faker.email())
    message_id = factory.LazyAttribute(lambda _: faker.uuid4())


class EventLogFactory(DjangoModelFactory):
    class Meta:
        model = EventLog

    email_activity_tracker = factory.SubFactory(EmailActivityTrackerFactory)
    event_type = factory.LazyAttribute(
        lambda _: faker.random_choices(
            elements=[x[0] for x in EventLog.EVENT_TYPE_CHOICES]
        )[0]
    )
    event_at = factory.LazyAttribute(lambda _: faker.date_time(tzinfo=pytz.utc))
    event_info = {"event": "sent"}



from django.contrib import admin
from nested_admin.nested import NestedTabularInline, NestedModelAdmin

from .models import EmailLog, EmailActivityTracker, EventLog


class NonEditableAdminMixin(object):
    def has_add_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return obj is None

    def has_delete_permission(self, request, obj=None):
        return False


class EventLogAdmin(NonEditableAdminMixin, NestedTabularInline):
    model = EventLog
    ordering = ("-event_at",)


class EmailActivityTrackerAdmin(NonEditableAdminMixin, NestedTabularInline):
    model = EmailActivityTracker
    inlines = [EventLogAdmin]


class EmailLogAdmin(NonEditableAdminMixin, NestedModelAdmin):
    inlines = [EmailActivityTrackerAdmin]
    list_display = (
        "id",
        "email_provider",
        "from_email",
        "to_emails",
        "subject",
        "template_id",
        "dispatch_status",
        "created_at",
    )
    list_filter = (
        "created_at",
        "dispatch_status",
        "email_provider",
        "from_email",
        "template_id",
        "subject",
    )
    search_fields = ("to_emails", "cc_emails", "bcc_emails")


admin.site.register(EmailLog, EmailLogAdmin)



from django.db import models
from django.db.models import Q
from django.db.models.query import QuerySet
from django.utils import timezone


class AbstractQuerySet(QuerySet):
    def update(self, **kwargs):
        if "updated_at" not in kwargs:
            kwargs["updated_at"] = timezone.now()
        return super().update(**kwargs)

    def delete(self):
        return self.update(is_active=False)

    def force_delete(self):
        return super().delete()


class AbstractManager(models.Manager):
    def get_queryset(self, enforce_filter=True):
        qs = AbstractQuerySet(self.model, using=self._db)
        return qs.filter(is_active=True) if enforce_filter else qs

    def unfiltered(self):
        return self.get_queryset(enforce_filter=False)


class AbstractModel(models.Model):
    objects = AbstractManager()

    created_at = models.DateTimeField(
        default=timezone.now,
        help_text="Time of creation of this object",
        editable=False,
    )
    updated_at = models.DateTimeField(
        auto_now=True, help_text="Time of updation of this object", editable=False
    )
    is_active = models.BooleanField(
        default=True,
        help_text="Denotes if the object is active or not. Inactive objects behave similar to how a deleted object "
        "works.",
    )

    class Meta:
        abstract = True
        indexes = [
            models.Index(
                fields=["is_active"],
                condition=Q(is_active=True),
                name="%(class)s_is_active",
            )
        ]

    def update_fields(self, **params):
        self.updated_at = timezone.now()
        update_fields = ["updated_at"]

        for key, value in params.items():
            setattr(self, key, value)
            update_fields.append(key)

        self.save(update_fields=update_fields)

    def delete(self, using=None, keep_parents=False):
        return self.update_fields(is_active=False)
